@@ -27,6 +27,17 @@ os.chdir(
     )
 )
 
+
+DISTRO_SPECS = os.environ.get(
+    'MITOGEN_TEST_DISTRO_SPECS',
+    'centos6 centos8 debian9 debian11 ubuntu1604 ubuntu2004',
+)
+IMAGE_TEMPLATE = os.environ.get(
+    'MITOGEN_TEST_IMAGE_TEMPLATE',
+    'public.ecr.aws/n5z0e8q9/%(distro)s-test',
+)
+
+
 _print = print
 def print(*args, **kwargs):
     file = kwargs.get('file', sys.stdout)
@@ -189,12 +200,6 @@ class Fold(object):
 
 
 GIT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
-# Used only when MODE=mitogen
-DISTRO = os.environ.get('DISTRO', 'debian9')
-# Used only when MODE=ansible
-DISTROS = os.environ.get('DISTROS', 'centos6 centos8 debian9 debian11 ubuntu1604 ubuntu2004').split()
-TARGET_COUNT = int(os.environ.get('TARGET_COUNT', '2'))
-BASE_PORT = 2200
 TMP = TempDir().path
 
 
@@ -217,6 +222,7 @@ os.environ['PYTHONPATH'] = '%s:%s' % (
 def get_docker_hostname():
     """Return the hostname where the docker daemon is running.
     """
+    # Duplicated in testlib
     url = os.environ.get('DOCKER_HOST')
     if url in (None, 'http+docker://localunixsocket'):
         return 'localhost'
@@ -225,27 +231,34 @@ def get_docker_hostname():
     return parsed.netloc.partition(':')[0]
 
 
-def make_containers(name_prefix='', port_offset=0):
+def container_specs(
+        distros,
+        base_port=2200,
+        image_template=IMAGE_TEMPLATE,
+        name_template='target-%(distro)s-%(index)d',
+):
     """
     >>> import pprint
-    >>> BASE_PORT=2200; DISTROS=['debian11', 'centos6']
-    >>> pprint.pprint(make_containers())
+    >>> pprint.pprint(container_specs(['debian11-py3', 'centos6']))
     [{'distro': 'debian11',
       'family': 'debian',
       'hostname': 'localhost',
       'image': 'public.ecr.aws/n5z0e8q9/debian11-test',
+      'index': 1,
       'name': 'target-debian11-1',
       'port': 2201,
-      'python_path': '/usr/bin/python'},
+      'python_path': '/usr/bin/python3'},
      {'distro': 'centos6',
       'family': 'centos',
       'hostname': 'localhost',
       'image': 'public.ecr.aws/n5z0e8q9/centos6-test',
+      'index': 2,
       'name': 'target-centos6-2',
       'port': 2202,
       'python_path': '/usr/bin/python'}]
     """
     docker_hostname = get_docker_hostname()
+    # Code duplicated in testlib.py, both should be updated together
     distro_pattern = re.compile(r'''
         (?P<distro>(?P<family>[a-z]+)[0-9]+)
         (?:-(?P<py>py3))?
@@ -256,30 +269,27 @@ def make_containers(name_prefix='', port_offset=0):
     i = 1
     lst = []
 
-    for distro in DISTROS:
+    for distro in distros:
+        # Code duplicated in testlib.py, both should be updated together
         d = distro_pattern.match(distro).groupdict(default=None)
-        distro = d['distro']
-        family = d['family']
-        image = 'public.ecr.aws/n5z0e8q9/%s-test' % (distro,)
 
-        if d['py'] == 'py3':
+        if d.pop('py') == 'py3':
             python_path = '/usr/bin/python3'
         else:
             python_path = '/usr/bin/python'
 
-        if d['count']:
-            count = int(count)
-        else:
-            count = 1
+        count = int(d.pop('count') or '1', 10)
 
         for x in range(count):
-            lst.append({
-                "distro": distro, "family": family, "image": image,
-                "name": name_prefix + ("target-%s-%s" % (distro, i)),
+            d['index'] = i
+            d.update({
+                'image': image_template % d,
+                'name': name_template % d,
                 "hostname": docker_hostname,
-                "port": BASE_PORT + i + port_offset,
+                'port': base_port + i,
                 "python_path": python_path,
             })
+            lst.append(d)
             i += 1
 
     return lst
